import os
import time
import logging
import json
import base64
import threading
from pymodbus.client.sync import ModbusSerialClient as ModbusClient
from gsmmodem.modem import GsmModem
from gsmmodem.exceptions import TimeoutException
from modules.modbus_exception import ModbusNoneResponseException
from constants.core_modules_info import LCBInfo
# from public.date_functions import get_current_time

LOGGER = logging.getLogger(__name__)


def refresh_modbus_client(func):
    fname = func.func_name

    def wrapper(self, *args, **kwargs):
        ret = None
        try:
            if self.stop_loop:
                LOGGER.debug("Stopped Receiving Modbus Request due to set of Stop signal")
            else:
                if not self.is_lcb_booted():
                    self.power_on()
                else:
                    LOGGER.debug("LCB already booted up")
                if self.is_lcb_booted():
                    self.client = ModbusClient(method=LCBInfo.METHOD, baudrate=LCBInfo.BAUDRATE,
                                               port=LCBInfo.PORT, timeout=LCBInfo.TIMEOUT)
                    for i in xrange(1, 3):
                        LOGGER.debug("Attempt {} for modbus call {}...".format(i, fname))
                        try:
                            # noinspection PyArgumentList
                            ret = func(self, *args, **kwargs)
                            if ret is None:
                                LOGGER.error("Modbus call {} returned none in attempt {}".format(fname, i))
                                continue
                            break
                        except Exception:
                            LOGGER.exception("Failed in attempt {} for modbus call {}...".format(str(i), fname))
                            if i == 2:
                                # raising error
                                raise
                            continue
                else:
                    LOGGER.info("LCB not booted")
        finally:
            try:
                if self.client:
                    self.client.close()
            except:
                LOGGER.warn("Failed to close Modbus Serial Port")

        return ret

    return wrapper


def modbus_sync(func):
    def wrapper(self, *args, **kwargs):
        if self.data_mutex._is_owned():
            LOGGER.debug("{} already has modbus mutex in modbus_sync".format(threading.currentThread().name))
            LOGGER.debug("Function name: {}".format(func.__name__))
            # noinspection PyArgumentList
            return func(self, *args, **kwargs)

        LOGGER.debug("{} trying to acquire low_lock".format(threading.currentThread().name))
        self.low_lock()
        LOGGER.debug("{} acquired low_lock".format(threading.currentThread().name))
        try:
            # noinspection PyArgumentList
            return func(self, *args, **kwargs)
        finally:
            self.low_unlock()
            LOGGER.debug("{} released low_lock".format(threading.currentThread().name))

    return wrapper


def modbus_sync_high(func):
    def wrapper(self, *args, **kwargs):
        if self.data_mutex._is_owned():
            LOGGER.debug("{} already has modbus mutex in modbus_sync_high".format(threading.currentThread().name))
            LOGGER.debug("Function name: {}".format(func.__name__))
            # noinspection PyArgumentList
            return func(self, *args, **kwargs)

        LOGGER.debug("{} trying to acquire high_lock".format(threading.currentThread().name))
        self.high_lock()
        LOGGER.debug("{} acquired high_lock".format(threading.currentThread().name))
        try:
            # noinspection PyArgumentList
            return func(self, *args, **kwargs)
        finally:
            self.high_unlock()
            LOGGER.debug("{} released high_lock".format(threading.currentThread().name))

    return wrapper


'''
========================================================================================================================
End: LCBModbus Decorators
========================================================================================================================
'''

'''
========================================================================================================================
Start: Modem Decorators
========================================================================================================================
'''


def modem_sync(func):
    fname = func.func_name

    def wrapper(self, *args, **kwargs):

        if not os.path.exists(self.modem_comm_port):
            LOGGER.warn("Modem not found")
            return None
        try:
            LOGGER.debug("Creating modem serial...")
            self.modem = GsmModem(self.modem_comm_port, 921600)
            LOGGER.debug("Modem serial created...")
            self.modem.log.setLevel(logging.ERROR)
            self.modem.connect()
            LOGGER.debug("Calling modem wrapped function: {}".format(fname))
            ret = func(self, *args, **kwargs)
            LOGGER.debug("Completed modem wrapped function: {}".format(fname))
        except TimeoutException:
            LOGGER.warn("Timeout exception for function: {}. Resetting modem...".format(fname))
            if self.reset():
                action = "Reset Modem"
            else:
                action = "No Action Taken"
            return None
        except Exception:
            LOGGER.exception("Error in modem connect for function: {}".format(fname))
            return None
        finally:
            if self.modem:
                self.modem.close()
            self.modem = None
            LOGGER.debug("Releasing modem lock...")
            # self.mutex.release()
            # LOGGER.debug("Released modem lock...")
        return ret

    return wrapper


'''
========================================================================================================================
End: Modem Decorators
========================================================================================================================
'''
