import mraa
import time
import logging
import threading
import os

from constants.core_modules_info import LCBInfo
from constants.common import CommonInfo
from util import config
from util.config import get_config_value, update_config_file

# pymodbus classes
from pymodbus.bit_read_message import ReadDiscreteInputsResponse
from pymodbus.bit_write_message import WriteMultipleCoilsResponse, WriteSingleCoilResponse
from pymodbus.register_read_message import ReadHoldingRegistersResponse, ReadInputRegistersResponse
from modules.modbus_exception import ModbusUnexpectedResponseException
from pymodbus.client.sync import ModbusSerialClient as ModbusClient

# DOOR_COUNT = 12
MAX_LCB_DOOR_COUNT = 12

LOGGER = logging.getLogger(__name__)


class LCBModbus:

    def __init__(self):

        self.booted = False
        self.modem = None
        self.lcb_info_collected = False
        self.modbus_reset_timeout = LCBInfo.LCB_OFF_TIMEOUT
        self.stop_loop = False
        self.bay_offset = LCBInfo.BAY_OFFSET
        self.state = None
        self.dc_power_state = None
        self.bay_door_state = None
        self.power_pin = mraa.Gpio(LCBInfo.POWER_PIN)
        self.dc_power_pin = mraa.Gpio(LCBInfo.DC_POWER_PIN)
        self.bay_door_pin = mraa.Gpio(LCBInfo.BAY_DOOR_PIN)
        self.modbus_timeout = self.modbus_reset_timeout
        self.power_gap = 0
        self.previous_power_off = 0
        self.raw_power = False
        self.client = ModbusClient(method=LCBInfo.METHOD, baudrate=LCBInfo.BAUDRATE,
                                   port=LCBInfo.PORT, timeout=LCBInfo.TIMEOUT)
        self.lcb_init()

    '''
    ====================================================================================================================
    Start: private methods
    ====================================================================================================================
    '''

    def _read_holding_registers(self, column, size, unit=LCBInfo.DEFAULT_SLAVE_NO):
        return self.client.read_holding_registers(int(column), int(size), unit=unit)

    def _read_input_registers(self, address, unit=LCBInfo.DEFAULT_SLAVE_NO):
        return self.client.read_input_registers(int(address), unit=unit)

    def _read_discrete_inputs(self, address, count=1, unit=LCBInfo.DEFAULT_SLAVE_NO):
        return self.client.read_discrete_inputs(int(address), int(count), unit=unit)

    def _write_single_coil_high(self, address, value, unit=LCBInfo.DEFAULT_SLAVE_NO):
        return self.client.write_coil(int(address), value, unit=unit)

    def _write_multiple_coils(self, address, value, unit=LCBInfo.DEFAULT_SLAVE_NO):
        return self.client.write_coils(int(address), value, unit=unit)

    '''
    ====================================================================================================================
    End: Private methods
    ====================================================================================================================
    '''

    def lcb_init(self):
        self.power_pin.dir(mraa.DIR_OUT)
        self.dc_power_pin.dir(mraa.DIR_IN)
        self.bay_door_pin.dir(mraa.DIR_IN)
        self.reset_modbus_timeout()
        self.power_off()
        if self.modbus_reset_timeout > 0:
            self.power_on()
        else:
            LOGGER.debug("No need of power on")
        self.door_count = int(get_config_value(filename=CommonInfo.POD_CONFIG_FILE, section="general", field="door_count"))
        print(self.door_count)

    def reset_modbus_timeout(self):
        self.modbus_timeout = self.modbus_reset_timeout

    # Gives True when LCB is ON else False
    def is_lcb_on(self):
        return bool(self.power_pin.read())

    def is_lcb_booted(self):
        return self.is_lcb_on() and self.booted

    @staticmethod
    def is_column_valid(column):
        assert (isinstance(column, int) and 0 < column <= LCBInfo.COLUMN_RANGE), "Invalid column"

    def read_lock_status(self, door_no, priority_high=False, door_count=1):
        response = self._read_discrete_inputs(address=door_no, count=door_count, unit=LCBInfo.DEFAULT_SLAVE_NO)
        if isinstance(response, ReadDiscreteInputsResponse):
            LOGGER.debug("Read lock status response: {}".format(response.bits))
            return response.bits[0:door_count]
        else:
            return None
            # raise ModbusUnexpectedResponseException(str(response))

    '''
    ====================================================================================================================
    Start: Door methods
    ====================================================================================================================
    '''

    def open_lock(self, door, key_uuid):
        opened = None
        try:
            LOGGER.info("Writing coil, key uuid: {}".format(key_uuid))
            response = self._write_single_coil_high(int(door), True)
            LOGGER.info("Got write response: {}".format(isinstance(response, WriteSingleCoilResponse)))
            if isinstance(response, WriteSingleCoilResponse):
                opened = response.value
            else:
                LOGGER.error("ModbusUnexpectedResponseException")
                return None
                # raise ModbusUnexpectedResponseException(str(response))
        except Exception:
            LOGGER.exception("Got exception while writing coil")
        return self.get_open_doors()

    '''
    ====================================================================================================================
    End: Door methods
    ====================================================================================================================
    '''
    '''
    ====================================================================================================================
    Start: Buzzer methods
    ====================================================================================================================
    '''

    def start_buzzer(self, column=1, frequency=1):
        if frequency == 0:
            value = [False, False]
        elif frequency == 1:
            value = [False, True]
        elif frequency == 2:
            value = [True, False]
        else:
            value = [True, True]
        address = LCBInfo.BUZZER_BASE_ADDRESS + (2 * (column - 1))
        response = self._write_multiple_coils(address=address, value=value, unit=LCBInfo.DEFAULT_SLAVE_NO)
        LOGGER.debug("Operate buzzer response: {}".format(response))
        if isinstance(response, WriteMultipleCoilsResponse):
            return response
        else:
            LOGGER.error("ModbusUnexpectedResponseException")
            return None
            # raise ModbusUnexpectedResponseException(str(response))

    def stop_buzzer(self, column=1):
        value = [False, False]
        address = LCBInfo.BUZZER_BASE_ADDRESS + (2 * (column - 1))
        response = self._write_multiple_coils(address=address, value=value, unit=LCBInfo.DEFAULT_SLAVE_NO)
        # response = self._write_multiple_coils(address=address, value=value, unit=LCBInfo.DEFAULT_SLAVE_NO)
        LOGGER.debug("Operate buzzer response: {}".format(response))
        if isinstance(response, WriteMultipleCoilsResponse):
            return response
        else:
            LOGGER.error("ModbusUnexpectedResponseException")
            return None
            # raise ModbusUnexpectedResponseException(str(response))

    '''
    ====================================================================================================================
    End: Buzzer methods
    ====================================================================================================================
    '''

    def get_serial_number(self, column=1):
        column = int(column)
        self.is_column_valid(column)
        response = self._read_holding_registers(column=column, size=3, unit=LCBInfo.DEFAULT_SLAVE_NO)
        if isinstance(response, ReadHoldingRegistersResponse):
            serial_number = ""
            for reg in response.registers:
                serial_number += str(hex(reg))[2:].decode("hex")
            return serial_number
        else:
            LOGGER.error("ModbusUnexpectedResponseException")
            return None
            # raise ModbusUnexpectedResponseException(str(response))

    def get_firmware_version(self, column=1):
        LOGGER.debug("In get_firmware_version method for column " + str(column))
        column = int(column)
        self.is_column_valid(column)
        response = self._read_input_registers(address=column, unit=LCBInfo.DEFAULT_SLAVE_NO)
        if isinstance(response, ReadInputRegistersResponse):
            firmware_version = "%04d" % int(
                str(hex(response.registers[0]))[2:])
            return firmware_version[:2] + '.' + firmware_version[2:]
        else:
            LOGGER.error("ModbusUnexpectedResponseException")
            return None
            # raise ModbusUnexpectedResponseException(str(response))

    def collect_lcb_info(self):
        LOGGER.debug("In collect LCB info...")
        if self.lcb_info_collected is False:
            LOGGER.debug("Collecting LCB info for first time now...")
            try:
                if self.is_lcb_booted():
                    section_name = "LCB"
                    config = ConfigParser.ConfigParser()
                    config.add_section(section_name)
                    config.set(section_name, "uid", self.get_serial_number())
                    config.set(section_name, "serial", self.get_serial_number())
                    config.set(section_name, "version", self.get_firmware_version())
                    config.write(open(LCB_INFO_FILE, 'w'))
                    self.lcb_info_collected = True
                else:
                    LOGGER.info("Info not collected as LCB is off")
            except Exception:
                LOGGER.exception("Error while collecting lcb version and serial")
            LOGGER.info("Out collect LCB info...")
        else:
            LOGGER.debug("LCB info already collected")

    def reset_lcb(self):
        try:
            self.power_pin.write(False)
            time.sleep(15)
        finally:
            self.power_pin.write(True)

    def power_on(self):
        LOGGER.debug("Inside power on ")
        self.reset_modbus_timeout()
        if not bool(self.power_pin.read()):
            LOGGER.debug("Powering on LCB")
            self.power_pin.write(True)
            mraa.Uart(1)
            LOGGER.debug("Waiting for LCB to power on")
            time.sleep(3)
            LOGGER.debug("LCB Powered on")
            self.booted = True

    def power_off(self):
        power_status = self.is_lcb_on()
        LOGGER.debug("Power status: {}".format(power_status))
        if power_status:
            self.power_pin.write(False)
            a = mraa.Gpio(45)
            b = mraa.Gpio(46)
            a.dir(mraa.DIR_OUT)
            b.dir(mraa.DIR_OUT)
            a.write(False)
            b.write(False)
            LOGGER.debug("LCB Powered off")
            self.booted = False

    def read_bay_door_status(self, column=1):
        is_open = bool(self.bay_door_pin.read())
        if is_open is False and (self.bay_door_state is None or self.bay_door_state is True):
            self.bay_door_state = False
        elif is_open is True and (self.bay_door_state is None or self.bay_door_state is False):
            self.bay_door_state = True
        return is_open

    def is_door_locked(self, door_no):
        door_no = int(door_no)
        if door_no < self.bay_offset:
            door_open_info = self.read_lock_status(door_no=door_no)
            if door_open_info:
                door_closed = not door_open_info[0]
            else:
                door_closed = False
            return door_closed
        else:
            return not self.read_bay_door_status(door_no - self.bay_offset)

    def read_dc_power_status(self):
        res = not bool(self.power_pin.read())
        if res is False and (self.dc_power_state is None or self.dc_power_state is True):
            self.dc_power_state = False
            return False
        elif res is True and (self.dc_power_state is None or self.dc_power_state is False):
            self.dc_power_state = True
            return True
        else:
            return res

    def get_open_doors(self, priority_high=False):
        LOGGER.debug("In modbus get open doors...")
        doors = []
        locks_status = self.read_lock_status(door_no=1, priority_high=priority_high, door_count=self.door_count)
        if locks_status is None:
            return None
        for index, status in enumerate(locks_status):
            if status:
                doors.append(index + 1)
        return doors

    def read_write_powergap(self, mode, value=0):
        power_section = "battery_status"
        power_file = CommonInfo.POD_CONFIG_FILE
        if mode == "read":
            self.power_gap = int(
                get_config_value(filename=power_file, section=power_section, field="battery_power_time"))
        if mode == "write":
            update_config_file(filename=power_file, section=power_section, field="battery_power_time", value=value)

    def is_raw_power_off(self):
        status = bool(self.dc_power_pin.read())
        self.read_write_powergap("read")
        if not status:
            self.power_gap = 0
            self.read_write_powergap("write", value=0)
            self.previous_power_off = 0
        elif self.previous_power_off == 0:
            self.previous_power_off = time.time()
        else:
            self.power_gap += int(time.time() - self.previous_power_off)
            self.read_write_powergap("write", value = self.power_gap)
            self.previous_power_off = time.time()
        return status, self.power_gap
