import mraa
import time
import logging
from threading import Lock

from constants.core_modules_info import LcdInfo

from util import command_utils
from util import config
from constants.common import CommonInfo

LOGGER = logging.getLogger(__name__)
DELAY = 0.001


class LcdHardware:
    def __init__(self):
        self.diag_disp = False
        self.configure_lcd()

    def configure_lcd(self):
        self.pod_name = config.get_config_value(filename=CommonInfo.POD_CONFIG_FILE, section="general", field="pod_name")
        if self.pod_name is "":
            self.onboard_flag = True
            self.pod_name = command_utils.output(
                "timeout -k 10 5 ifconfig ra0 | grep HWaddr | cut -d ' ' -f 12 | tr -d : | tail -c 7")
        else:
            self.onboard_flag = False

        self.lcd_lock = Lock()
        # Assigning Linkit pins
        self.RS = mraa.Gpio(LcdInfo.LCD_RS)
        self.E = mraa.Gpio(LcdInfo.LCD_E)
        self.D4 = mraa.Gpio(LcdInfo.LCD_D4)
        self.D5 = mraa.Gpio(LcdInfo.LCD_D5)
        self.D6 = mraa.Gpio(LcdInfo.LCD_D6)
        self.D7 = mraa.Gpio(LcdInfo.LCD_D7)
        self.BL = mraa.Gpio(LcdInfo.LCD_BL)
        self.blink_flag = False

        # Configuring the pins as output pins
        self.RS.dir(mraa.DIR_OUT)
        self.E.dir(mraa.DIR_OUT)
        self.D4.dir(mraa.DIR_OUT)
        self.D5.dir(mraa.DIR_OUT)
        self.D6.dir(mraa.DIR_OUT)
        self.D7.dir(mraa.DIR_OUT)
        self.BL.dir(mraa.DIR_OUT)

    def set_backlight(self, power):
        self.BL.write(power)

    def get_backlight_status(self):
        return self.BL.read()

    def lcd_byte(self, bits, mode):
        self.RS.write(mode)

        self.D4.write(False)
        self.D5.write(False)
        self.D6.write(False)
        self.D7.write(False)
        if bits & 0x10 == 0x10:
            self.D4.write(True)
        if bits & 0x20 == 0x20:
            self.D5.write(True)
        if bits & 0x40 == 0x40:
            self.D6.write(True)
        if bits & 0x80 == 0x80:
            self.D7.write(True)

        self.lcd_toggle_enable()

        self.D4.write(False)
        self.D5.write(False)
        self.D6.write(False)
        self.D7.write(False)
        if bits & 0x01 == 0x01:
            self.D4.write(True)
        if bits & 0x02 == 0x02:
            self.D5.write(True)
        if bits & 0x04 == 0x04:
            self.D6.write(True)
        if bits & 0x08 == 0x08:
            self.D7.write(True)

        self.lcd_toggle_enable()

    def lcd_toggle_enable(self):
        self.E.write(True)
        time.sleep(DELAY)
        self.E.write(False)
        time.sleep(DELAY)

    def initialize_lcd(self):
        self.lcd_byte(0x33, LcdInfo.LCD_CMD)
        self.lcd_byte(0x32, LcdInfo.LCD_CMD)
        self.lcd_byte(0x06, LcdInfo.LCD_CMD)
        self.lcd_byte(0x0C, LcdInfo.LCD_CMD)
        self.lcd_byte(0x28, LcdInfo.LCD_CMD)
        self.lcd_byte(0x01, LcdInfo.LCD_CMD)
        time.sleep(DELAY)

    @staticmethod
    def get_line(line):
        line = int(line)
        if line == 1:
            return LcdInfo.LCD_LINE_1
        elif line == 2:
            return LcdInfo.LCD_LINE_2
        elif line == 3:
            return LcdInfo.LCD_LINE_3
        elif line == 4:
            return LcdInfo.LCD_LINE_4
        else:
            LOGGER.warn("Line {} not found".format(line))
            return None

    def lcd_write_string(self, message, line, style_type):
        if style_type == "left":
            message = message.ljust(LcdInfo.LCD_WIDTH, " ")
        elif style_type == "center":
            message = message.center(LcdInfo.LCD_WIDTH, " ")
        elif style_type == "right":
            message = message.rjust(LcdInfo.LCD_WIDTH, " ")
        else:
            LOGGER.warn("Style {} not recognized".format(style_type))
            return

        line = int(line)
        if line == 1:
            line_addr = LcdInfo.LCD_LINE_1
        elif line == 2:
            line_addr = LcdInfo.LCD_LINE_2
        elif line == 3:
            line_addr = LcdInfo.LCD_LINE_3
        elif line == 4:
            line_addr = LcdInfo.LCD_LINE_4
        else:
            LOGGER.warn("Line {} not found".format(line))
            return None

        self.lcd_byte(line_addr, LcdInfo.LCD_CMD)

        for i in range(LcdInfo.LCD_WIDTH):
            self.lcd_byte(ord(message[i]), LcdInfo.LCD_CHR)

    def blink(self, frequency, count=1):
        for i in range(count * 2):
            self.set_backlight(not bool(self.get_backlight_status()))
            time.sleep(frequency)
        time.sleep(1)
        self.set_backlight(1)

    def lcd_diagnostics(self):
        self.set_backlight(1)
        self.lcd_clear(1, 0)
        self.lcd_clear(2, 0)
        self.lcd_clear(3, 0)
        self.lcd_clear(4, 0)
        time.sleep(1)
        self.lcd_settextline("Line 1 Left", 1, "left", True)
        self.lcd_settextline("Line 2 Center", 2, "center", True)
        self.lcd_settextline("Line 3 Right", 3, "right", True)
        self.lcd_settextline("Line 4 Left", 4, "left", True)
        """
        self.lcd_settextline(" _   |  _  _  _|", 1, "left", True)
        self.lcd_settextline("| | .|/| || || |", 2, "left", True)
        self.lcd_settextline(" -| l|\|-  -  -", 3, "left", True)
        self.lcd_settextline("  |/   |      ", 4, "left", True)
        """
        time.sleep(3)
        self.lcd_backlight_blink(2, count=2)
        time.sleep(3)
        self.initialize_lcd()
        self.lcd_settextline("Starting...", 2, "center", True)

    def lcd_reset(self):
        self.configure_lcd()
        return

    def lcd_settextline(self, sText, iLineNo, sStyle, bClear):
        if bClear:
            self.lcd_write_string("", iLineNo, "center")
        self.lcd_write_string(sText, iLineNo, sStyle)

    def lcd_clear(self, iLineNo, WaitSec):
        time.sleep(WaitSec)
        self.lcd_settextline("", iLineNo, "center", True)

    def lcd_backlight_state(self):
        return self.get_backlight_status()

    def lcd_backlight_blink(self, iFreq, count=1):
        self.blink(iFreq, count=count)

    # Research Later
    def lcd_toggle_state(self, bState):
        return

    def lcd_init_message(self):
        if self.diag_disp:
            self.pod_name = "Pod Testing"
        self.initialize_lcd()
        self.lcd_settextline("Qp-" + self.pod_name, 1, "center", True)
        if self.onboard_flag:
            self.lcd_settextline('Enter The Pod ID', 2, "center", True)
        else:
            self.lcd_settextline('Enter Your OTP', 2, "center", True)
        self.lcd_settextline("_" * 6, 3, "center", True)
