import os
import re
import mraa
import threading
import time
import json
import base64
import logging
import itertools
from decorators import modem_sync
from util.config import get_config_value
from constants.common import ModemInfo, CommonInfo

from util import command_utils

LOGGER = logging.getLogger(__name__)


class ModemPlugin:

    def __init__(self):
        self.modem_mode_config = ModemInfo.MODEM_MODE_CONFIG
        self.known_service_providers = [ModemInfo.AIRTEL_NAME, ModemInfo.TATA_DOCOMO_NAME,
                                        ModemInfo.VODAFONE_NAME]
        self.ussd_code_map = ModemInfo.USSD_CODE_MAP
        self.modem_comm_port = self.__get_modem_common_config_value(field=ModemInfo.MODEM_COMM_PORT_NAME)
        self.modem_data_port = self.__get_modem_common_config_value(field=ModemInfo.MODEM_DATA_PORT_NAME)
        # Configurations
        self.preferred_network_type = self.__get_modem_common_config_value(
            field=ModemInfo.PREFERRED_NETWORK_TYPE_NAME)
        # Modem instance variable
        self.modem = None
        # Modem power pin
        self.power_pin = mraa.Gpio(int(ModemInfo.MODEM_POWER_PIN))
        self.power_pin.dir(mraa.DIR_OUT)
        self.reset_rr = itertools.cycle([self.soft_reset, self.hard_reset, self.power_reset])
        self.modem_configured = False

    """
    ====================================================================================================================
    Start: Private/Protected methods
    ====================================================================================================================
    """

    @staticmethod
    def __get_modem_common_config_value(field, default=None):
        return get_config_value(filename=CommonInfo.POD_CONFIG_FILE,
                                section=ModemInfo.CONFIG_SECTION_NAME,
                                field=field, default=default)

    def __set_network_mode(self):
        try:
            qmi_device = ModemInfo.QMI_DEVICE_NAME
            LOGGER.info('Qmi device: %s', qmi_device)
            if self.preferred_network_type == "auto":
                qmi_network_type = "all"
            else:
                qmi_network_type = self.preferred_network_type
            if os.path.exists(qmi_device):
                LOGGER.info("Setting qmi modem network to {}".format(qmi_network_type))
                try:
                    status, output = command_utils.status_output(
                        "uqmi -d {} --set-network-modes {}".format(qmi_device, qmi_network_type))
                    if status != 0:
                        LOGGER.error("Exception in setting network mode: {}".format(output))
                    else:
                        LOGGER.info("Network mode for dongle set successfully")
                except:
                    LOGGER.exception("Exception in setting network mode")
            else:
                LOGGER.info("qmi modem network already set")
        except Exception as e:
            LOGGER.error("Error occurred in __set_network_mode function: %s", e)
            raise Exception(e)

    def __get_attached_modem(self):
        LOGGER.debug("In get attached modem...")
        try:
            supported_modems_info = self.__get_modem_common_config_value(
                field=ModemInfo.SUPPORTED_MODEMS_NAME)
            supported_modems = supported_modems_info.split(",")
            for modem in supported_modems:
                LOGGER.debug("Checking if modem with vid:pid {} attached...".format(modem))
                status = command_utils.status("lsusb | grep {}".format(modem))
                if status == 0:
                    LOGGER.debug("Return attached modem {}".format(modem))
                    return modem
                else:
                    continue
            LOGGER.warn("No attached modem found...")
            return None
        except Exception as e:
            LOGGER.exception("Exception in getting attached modem")
            raise Exception(e)

    def __write_to_modem(self, at_command):
        return self.modem.write(at_command)

    @modem_sync
    def __set_huawei_network_mode(self):
        try:
            LOGGER.info("Setting network mode to {} for Huawei modem...".format(self.preferred_network_type))
            if self.preferred_network_type in self.modem_mode_config:
                command = str(self.modem_mode_config.get(self.preferred_network_type))
                res = self.modem.write(command)
                LOGGER.info("Modem write response: {}".format(res))
                if len(res) == 2 and res[1] == 'OK':
                    LOGGER.info("Network mode set successfully...")
                    return res[0]
                else:
                    return None
            else:
                LOGGER.warn("Unsupported network type found in configuration: {}".format(self.preferred_network_type))
                return None
        except Exception as e:
            LOGGER.error("Error occurred in __set_huawei_network_mode function: %s", e)
            raise Exception(e)

    def __insert_modem_device_events(self, problem_type, status, resolution="No Action Taken", details=""):
        payload = {"resolution": resolution, "details": details}
        payload_encoded = base64.b64encode(json.dumps(payload))
        values_dict = {
            "TYPE": problem_type,
            "PAYLOAD": payload_encoded,
            "ACK": 0,
            "STATUS": status
        }
        return values_dict

    """
    ====================================================================================================================
    End: Private/Protected methods
    ====================================================================================================================

    """

    def plugin_start(self):
        self.power_pin.dir(mraa.DIR_OUT)
        self.power_pin.write(False)
        # Checking and setting network mode
        qmi_lock_file = ModemInfo.QMI_LOCK_FILE
        try:
            if os.path.exists(qmi_lock_file):
                self.__set_network_mode()
                os.remove(qmi_lock_file)
            else:
                LOGGER.debug("Network mode already set")

            e3373_lock = ModemInfo.E3373_LOCK_FILE
            if os.path.exists(e3373_lock):
                modem = self.__get_attached_modem()
                if modem == "12d1:1506":
                    LOGGER.info("Huawei modem with vid:pid {} found, setting network mode...".format(modem))
                    self.__set_huawei_network_mode()
                    os.remove(e3373_lock)
                else:
                    LOGGER.info("Ignoring other Modems")
            else:
                LOGGER.info("No lock exists for setting network changes")
            self.modem_configured = True
        except Exception as e:
            LOGGER.info("Error occurred while configuring Modem")
            # self.qikpod_manager.restart_plugin(plugin_name=PluginNames.MODEM)

    def plugin_stop(self):
        self.modem_configured = False
        command_utils.status("touch {}".format(ModemInfo.E3373_LOCK_FILE))

    @modem_sync
    def get_signal_strength(self):
        # Signal Strength Conversion
        if self.modem.signalStrength >= 0:
            strength = -111 + (2 * self.modem.signalStrength)
        else:
            strength = self.modem.signalStrength

        # Categorizing strength of signal
        if strength is None:
            return None
        elif strength == -1:
            quality = "Poor"
        elif strength < -93:
            quality = "Marginal"
        elif strength < -83:
            quality = "Ok"
        elif strength < -73:
            quality = "Good"
        elif strength >= -73:
            quality = "Excellent"
        else:
            quality = "Unknown"
        return str(strength) + "dBM:" + quality

    @modem_sync
    def get_imei_number(self):
        return self.modem.imei

    @modem_sync
    def get_imsi_number(self):
        return self.modem.imsi

    @modem_sync
    def get_service_provider(self):
        service_provider = str(self.modem.networkName.upper())
        if service_provider.startswith(ModemInfo.AIRTEL_NAME):
            service_provider = ModemInfo.AIRTEL_NAME
        elif ModemInfo.VODAFONE_NAME in service_provider:
            service_provider = ModemInfo.VODAFONE_NAME
        elif ModemInfo.TATA_DOCOMO_NAME in service_provider:
            service_provider = ModemInfo.TATA_DOCOMO_NAME
        else:
            LOGGER.debug("Internet Service Provider not recognised")
        return service_provider
    @modem_sync
    def get_network_mode(self):
        manufacturer = re.sub('^\+CGMI:[\s]', '', self.modem.manufacturer)
        if manufacturer:
            manufacturer = manufacturer.lower()
            if ModemInfo.HUAWEI_NAME in manufacturer:
                command = ModemInfo.HUAWEI_NETWORK_MODE_COMMAND
            else:
                command = ModemInfo.OTHER_NETWORK_MODE_COMMAND

            res = self.modem.write(command)
            LOGGER.info(res)
            if res and len(res) == 2 and res[1] == 'OK':
                return res[0]

        return None

    def get_sim_number(self):

        def _crsm_to_sim_no(crsm_values):
            def _splitter(values, mod_result):
                value_length = len(values)
                counter = 0
                new_values = ""
                for value in values:
                    if counter % 2 == mod_result:
                        new_values = new_values + value
                    counter += 1
                    if counter >= value_length:
                        break
                return new_values

            def _joiner(length, s1, s2):
                joined = ""
                for i in xrange(0, length):
                    joined = joined + s1[i] + s2[i]
                return joined

            set1 = _splitter(crsm_values, 1)
            set2 = _splitter(crsm_values, 0)
            assert (len(set1) == len(set2))
            return _joiner(len(set1), set1, set2)

        @modem_sync
        def _get_crsm_number(self):
            res = self.modem.write("AT+CRSM=176,12258,0,0,10")
            if len(res) == 2 and res[1] == 'OK':
                return res[0]
            else:
                return None

        ret = _get_crsm_number(self)
        if ret:
            return _crsm_to_sim_no(ret[ret.index('"') + 1:ret.rindex('"')])
        else:
            return None

    @modem_sync
    def soft_reset(self):
        try:
            if os.path.exists(self.modem_comm_port) and os.path.exists(self.modem_data_port):
                self.__write_to_modem(at_command=ModemInfo.SOFT_RESET_COMMAND)
                return True
        except Exception:
            LOGGER.exception("Exception in soft reset modem...")

        return False

    def power_reset(self):
        try:
            self.power_pin.write(True)
            time.sleep(15)
            self.power_pin.write(False)
            return True
        except Exception:
            LOGGER.exception("Exception in power reset")
        return False

    def power_off(self):
        try:
            self.power_pin.write(True)
        except Exception:
            LOGGER.exception("Cannot turn modem of")

    def hard_reset(self):
        try:
            attached_modem = self.__get_attached_modem()
            if attached_modem is None:
                LOGGER.warn("No attached modem so exiting hard reset call")
                return False
            status, output = command_utils.status_output("usbreset {}".format(attached_modem))
            if status == 0:
                return True
            else:
                LOGGER.error("Hard reset failed with error code {} and text {}".format(status, output))
                return False
        except Exception:
            LOGGER.exception("Error in hard reset")
        return False

    def reset(self):
        reset_function = self.reset_rr.next()
        LOGGER.info("Reset requested for modem using method {}".format(reset_function.func_name))
        try:
            LOGGER.info("Resetting using function: {}".format(reset_function.func_name))
            reset_status = reset_function()
            LOGGER.info("Reset {} using function {}".format(reset_status, reset_function.func_name))
            return "{}:{}".format(reset_function.func_name, reset_status)
        except Exception as e:
            LOGGER.exception("Reset Failed using {}: {}".format(reset_function.func_name, e.message))
            return "{}:FAIL:{}".format(reset_function.func_name, e.message)

    def get_modem_serial(self):
        res = self.__write_to_modem(at_command=ModemInfo.GET_SERIAL_COMMAND)
        if len(res) == 2 and res[1] == 'OK':
            return res[0]
        else:
            return None

    # This method response is used by ussdmanger modem test
    def status(self, do_ping_test=True):
        LOGGER.debug("In modem status check...")
        # present_time = get_current_time()
        # Check if modem exists and listed in lsusb
        modem_found = self.__get_attached_modem()
        if modem_found:
            # Check if modem exists and both required symlinks exists
            comm_exists = os.path.exists(self.modem_comm_port)
            data_exists = os.path.exists(self.modem_data_port)
            if comm_exists and data_exists:
                if do_ping_test:
                    ping_status = False
                    timeout = ModemInfo.PING_TIMEOUT
                    try:
                        for i in range(1, 6):
                            LOGGER.debug("Performing ping {} with timeout {}".format(i, timeout))
                            status = command_utils.status(command="ping -I 3g-3g -c 1 -W {} 8.8.8.8".format(timeout),
                                                          timeout=timeout + 10)
                            if status == 0:
                                ping_status = True
                                break
                    except Exception:
                        LOGGER.exception("Exception in checking modem status...")

                    if ping_status:
                        LOGGER.debug("Modem Status Success")
                        temp_dict = {"status": "Modem Pass"}
                        return True, temp_dict
                    else:
                        action = self.reset()
                        temp_dict = self.__insert_modem_device_events(problem_type="HARDWARE_NETWORK",
                                                                      status="Modem Ping Fail",
                                                                      resolution=action)
                        return False, temp_dict
                else:
                    LOGGER.debug("Skipping ping test as part of modem test")
                    return True, "Modem Pass"
            else:
                action = self.reset()
                temp_dict = self.__insert_modem_device_events(problem_type="HARDWARE_NETWORK",
                                                              status="Modem Ping Fail",
                                                              resolution=action)
                return False, temp_dict
        else:
            # Modem not found
            LOGGER.warn("Modem not connected, power reset and report to server now...")
            power_reset_status = self.power_reset()
            action = "No action taken"
            if power_reset_status:
                action = "Power Reset"
            temp_dict = self.__insert_modem_device_events(problem_type="HARDWARE_NETWORK",
                                                          status="Modem Ping Fail",
                                                          resolution=action)
            return False, temp_dict

    @modem_sync
    def get_location(self):
        try:
            self.__write_to_modem(at_command="AT+CREG=2")
            res = self.__write_to_modem(at_command="AT+CREG?")
            self.__write_to_modem(at_command="AT+CREG=0")
            if len(res) == 2 and res[1] == 'OK':
                match = re.compile(r'\+CREG:.*,.*,"(.*)","(.*)".*$').match(res[0])
                if match and len(match.groups()) == 2:
                    return str(match.group(1) + "," + match.group(2))
        except:
            LOGGER.exception("Error while getting network location")

        return None


