from optparse import OptionParser
import os.path
import struct
import sys

import serial

from modbus_tk import LOGGER  # @UnresolvedImport
from modbus_tk.hooks import call_hooks  # @UnresolvedImport
from modbus_tk.modbus import Master  # @UnresolvedImport
from modbus_tk.modbus_rtu import RtuMaster  # @UnresolvedImport
from modbus_tk.utils import threadsafe_function, get_log_buffer  # @UnresolvedImport


from modbus_tk.exceptions import (
    ModbusError, ModbusFunctionNotSupportedError,  ModbusInvalidResponseError  # @UnresolvedImport
)


PORT = "/dev/tty.SLAB_USBtoUART"
BAUD_RATE = 9600
SLAVE = 0x1
TIMEOUT = 3

CHUNK_SIZE = 128

INIT_FIRMWARE_UPGRADE = 0x65
WRITE_FIRMWARE_UPGRADE = 0x66


# Start MONKEY PATCHING
ORIG_MASTER_EXECUTE = Master.execute


@threadsafe_function
def execute(self, slave, function_code, starting_address, quantity_of_x=0, output_value=0, data_format="",
            expected_length=-1):
    try:
        return ORIG_MASTER_EXECUTE(self, slave, function_code, starting_address,
                                   quantity_of_x, output_value, data_format, expected_length)
    except ModbusFunctionNotSupportedError:
        pdu = ""
        is_read_function = False
        nb_of_digits = 0

        # open the connection if it is not already done
        self.open()

        if function_code == INIT_FIRMWARE_UPGRADE:
            fmt = ">BH" + ("H" if output_value >= 0 else "h")
            pdu = struct.pack(fmt, function_code, starting_address, output_value)
            if not data_format:
                data_format = ">HH"
            if expected_length < 0:
                # No length was specified and calculated length can be used:
                # slave + func + adress1 + adress2 + value1+value2 + crc1 + crc2
                expected_length = 8
        elif function_code == WRITE_FIRMWARE_UPGRADE:
            print("here")
        else:
            raise ModbusFunctionNotSupportedError("The {0} function code is not supported. ".format(function_code))

        # instantiate a query which implements the MAC (TCP or RTU) part of the protocol
        query = self._make_query()

        # add the mac part of the protocol to the request
        request = query.build_request(pdu, slave)

        # send the request to the slave
        retval = call_hooks("modbus.Master.before_send", (self, request))
        if retval is not None:
            request = retval
        if self._verbose:
            LOGGER.debug(get_log_buffer("-> ", request))
        self._send(request)

        call_hooks("modbus.Master.after_send", (self, ))

        if slave != 0:
            # receive the data from the slave
            response = self._recv(expected_length)
            retval = call_hooks("modbus.Master.after_recv", (self, response))
            if retval is not None:
                response = retval
            if self._verbose:
                LOGGER.debug(get_log_buffer("<- ", response))

            # extract the pdu part of the response
            response_pdu = query.parse_response(response)

            # analyze the received data
            (return_code, byte_2) = struct.unpack(">BB", response_pdu[0:2])

            if return_code > 0x80:
                # the slave has returned an error
                exception_code = byte_2
                raise ModbusError(exception_code)
            else:
                if is_read_function:
                    # get the values returned by the reading function
                    byte_count = byte_2
                    data = response_pdu[2:]
                    if byte_count != len(data):
                        # the byte count in the pdu is invalid
                        raise ModbusInvalidResponseError(
                            "Byte count is {0} while actual number of bytes is {1}. ".format(byte_count, len(data))
                        )
                else:
                    # returns what is returned by the slave after a writing function
                    data = response_pdu[1:]

                # returns the data as a tuple according to the data_format
                # (calculated based on the function or user-defined)
                print data_format
                print data
                result = struct.unpack(data_format, data)
                if nb_of_digits > 0:
                    digits = []
                    for byte_val in result:
                        for _ in range(8):
                            if len(digits) >= nb_of_digits:
                                break
                            digits.append(byte_val % 2)
                            byte_val >>= 1
                    result = tuple(digits)
                return result

# End MONKEY PATCHING
Master.execute = execute


def firmware_upgrade_intent(file_path, version):
    # Version is string in this format 0102 if version is 1.2 use string format
    master = RtuMaster(serial.Serial(port=PORT, baudrate=BAUD_RATE, bytesize=8, parity='N', stopbits=1))
    master.set_timeout(TIMEOUT)
    response = master.execute(
        SLAVE, INIT_FIRMWARE_UPGRADE, os.path.getsize(file_path), output_value=version)
    print repr(response)
    # TODO:Parse response for success or error
    return response


def _firmware_chunk_write(packet_count, chunk):
    master = RtuMaster(serial.Serial(port=PORT, baudrate=BAUD_RATE, bytesize=8, parity='N', stopbits=1))
    master.set_timeout(TIMEOUT)
    response = master.execute(SLAVE, WRITE_FIRMWARE_UPGRADE, packet_count, output_value=chunk)
    print repr(response)
    # TODO:Parse response for success or error
    return response


def firmware_upgrade(file_path):
    firmware_file = None
    try:
        firmware_file = open(file_path, "rb")
        success = True
        packet_count = 0
        while True:
            chunk = firmware_file.read(CHUNK_SIZE)
            if not chunk:
                # File reading completed
                break
            packet_count += 1
            if not _firmware_chunk_write(packet_count, chunk):
                # Chunk Write Failed
                print("Failed to write packet no {}".format(packet_count))
                success = False
                break
    finally:
        firmware_file.close()
    return success

if __name__ == "__main__":
    parser = OptionParser()
    parser.add_option("-f", "--file", dest="filepath",
                      help="Firmware file path (absolute file path)", metavar="FILE")
    parser.add_option("-v", "--version", dest="version",
                      help="Firmware version in dot notation E.g 1.2", metavar="VERSION")
    (options, args) = parser.parse_args()
    (options, args) = parser.parse_args()
    if not options.filepath:
        parser.error("Firmware file path not given")
        sys.exit(1)
    elif not options.version:
        parser.error("Firmware version not given")
        sys.exit(1)
    firmware_file_path = options.filepath
    if not os.path.exists(firmware_file_path):
        print("Path '{}' does not exists.".format(firmware_file_path))
        sys.exit(2)
    firmware_version = options.version
    try:
        major, minor = firmware_version.split(".")
        upgrade_intent_response = firmware_upgrade_intent(firmware_file_path, "%02d" % int(major) + "%02d" % int(minor))
        # if success start firmware upgrade
        # upgrade_response = firmware_upgrade(firmware_file_path)
    except ValueError:
        print("Major and Minor version should be dot notation format E.g 1.2")
        sys.exit(3)
