#!/usr/bin/python

# Test code for RTC IC PCF85263A
# LinkIt Smart 7688 board
# Company : Implantaire Technologies
# Author : Sowmya BR

# ------------------------
# Wiring details:
# Supply - 3.3V
# SDA    - GPIO_5
# SCL    - GPIO_4
# -------------------------

import time

import mraa  # @UnresolvedImport


# Slave address of RTC IC PCF85263A is 0xA2
# Address set as 7 bit: 01010001b corresponds to 0x51 after 1-bit
# shift-right as required for LinkIt Smart 7688 board (7-bit address)
RTC_SLAVE_ID = 0x51

# RTC registers
RTC_100SEC_REG = 0x00                  # milliseconds (100th sec) register
RTC_SECS_REG = 0x01                  # Seconds register (0 to 59)
RTC_MINS_REG = 0x02                  # Minutes register (0 59)
RTC_HOURS_REG = 0x03                  # Hours register (0 to 23)
RTC_DAYS_REG = 0x04                  # Date register (0 to 31)
# Day of week register (0 to 5, SUNDAY to SATURDAY)
RTC_WEEKDAYS_REG = 0x05
RTC_MONTHS_REG = 0x06                  # Month register (0 to 11)
RTC_YEARS_REG = 0x07                  # Year register (xyz)

RTC_FUNCTION_REG = 0x28                  # RTC Control register
# RAM byte register, used for date validation (battery backed)
RTC_RAM_REG = 0x2C
# Register to hold/stop the RTC while setting the date & time
RTC_STOP_ENA_REG = 0x2E
# Prescalar register clear - to ensure the accuracy of the time while
# setting date & time
RTC_RESET_REG = 0x2F


class RTC:

    def __init__(self, secs, mins, hours, days, dow, months, years):
        self.secs = secs
        self.mins = mins
        self.hours = hours
        self.days = days
        self.dow = dow
        self.months = months
        self.years = years

# Initialise I2C0
i2c = mraa.I2c(0)

# Function to initalise RTC


def rtc_init():
    i2c.writeReg(RTC_FUNCTION_REG, 0x00)				# Set RTC mode

# Sets the Time and Date to RTC
# Store the seconds value in RTC RAM byte
# The parameters(values) should be in BCD format
# The parameter  - seconds, minutes, hours, day of week, month, year

# Added by Shashank to be able to pass simple int value to set function


def convert_to_hex(value):
    return int("0x" + str(value), 16)


def convert_to_int(value):
    return int(hex(value)[2:][:-1])


def print_time():
    month_dict={
        1: "Jan", 2: "Feb", 3: "Mar", 4: "Apr", 5: "May", 6: "Jun",
        7: "Jul", 8: "Aug", 9: "Sep", 10: "Oct", 11: "Nov", 12: "Dec"
        }
    rtc = read_date_time()
    time = "%d:%d:%d" % (rtc.hours, rtc.mins, rtc.secs)
    date = "%d-%s-%d " % (rtc.years, rtc.months ,rtc.days )
    str = date+time   
    # print(str)
    return str

# All the references to convert_to_hex added by Shashank, so calling
# function can pass simple int values for the fields


def set_date_time(rtc):
    # Stop the RTC
    i2c.writeReg(RTC_STOP_ENA_REG, 1)
    # Set the seconds to RTC seconds register
    i2c.writeReg(RTC_SECS_REG, convert_to_hex(rtc.secs))
    # Set the minutes to RTC minutes register
    i2c.writeReg(RTC_MINS_REG, convert_to_hex(rtc.mins))
    # Set the hours to RTC hours register
    i2c.writeReg(RTC_HOURS_REG, convert_to_hex(rtc.hours))
    # Set the day to RTC day register
    i2c.writeReg(RTC_DAYS_REG, convert_to_hex(rtc.days))
    # Set the weekday to RTC day of week register
    i2c.writeReg(RTC_WEEKDAYS_REG, convert_to_hex(rtc.dow))
    # Set the month to RTC months register
    i2c.writeReg(RTC_MONTHS_REG, convert_to_hex(rtc.months))
    # Set the year to RTC year register
    i2c.writeReg(RTC_YEARS_REG, convert_to_hex(rtc.years))
    # Start the RTC
    i2c.writeReg(RTC_STOP_ENA_REG, 0)
    # Store desired constant (ex: 0xAA) to RAM, to validate the date & time
    i2c.writeReg(RTC_RAM_REG, 0xAA)

    print("*************************************")

    print ("Date & Time is set to:")
    # print_time(rtc)

    print ("*************************************")

# Function to read the RTC time and date
# The returned values will be in BCD format


def read_date_time():
    # Read the seconds register
    secs = i2c.readReg(RTC_SECS_REG)
    # Read the minutes register
    mins = i2c.readReg(RTC_MINS_REG)
    # Read the hours register
    hrs = i2c.readReg(RTC_HOURS_REG)
    # Read the day register
    day = i2c.readReg(RTC_DAYS_REG)
    # Read the day of week register
    wday = i2c.readReg(RTC_WEEKDAYS_REG)
    # Read the month register
    mon = i2c.readReg(RTC_MONTHS_REG)
    # Read the year register
    year = i2c.readReg(RTC_YEARS_REG)
    rtc = RTC(convert_to_int((secs & 0x7F)), convert_to_int((mins & 0x7F)), convert_to_int(hrs),
              convert_to_int((day & 0x3F)), convert_to_int(wday), convert_to_int((mon & 0x1F)), convert_to_int(year))
    # print_time(rtc)
    return rtc

# Function to check the validity of the date & time
# The returned values will be in BCD format
# Shashank: Added return true and false so as to use from my utility


def is_time_date_valid():
    # The RAM value will be written with 0xAA (this example uses 0xAA,
    # however, this can be any one byte value, except 0x00 or 0xFF), everytime
    # the  RTC time is written with the valid Time & Date
    ram = i2c.readReg(RTC_RAM_REG)

    if ram != 0xAA:
        print ("Invalid Time & Date and the battery may have been removed during the power absence ")
        return False

    else:
        print ("The RTC had valid Time & Date")
        return True


# Main function of the application
def main():
    print( "************* RTC demo **************")
    is_time_date_valid()						 # Check if the RTC has valid time & date

    print("Current Time:")
    read_date_time()

    # The below values are for the date & time of 10-12-2016, Saturday &
    # 22:30:40. The desired date & time can be entered below, to sync it with
    # RTC, using this demo code

    rtc = RTC(20, 30, 19, 16, 1, 5, 16)

    print ("Syncing the RTC with the provided Time & Date ")
    set_date_time(rtc)

    while True:
        print ("********************************")
        # Read the Date & Time
        read_date_time()
        print ("********************************")
        print (" ")
        print (" ")
        time.sleep(1)

# Set the slave ID to I2C
i2c.address(RTC_SLAVE_ID)
# Initialise the RTC
rtc_init()

# Call main function until keyboard interrupts
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass
