#!/bin/sh

LOG_FILE="$(pwd)/logs/monit.log"
WATCH_DOG_FILE="/tmp/qikpod-service.run"

log() {
  timeout -k 10 10 echo "$(date): $1" | tee -a $LOG_FILE
}

check_p1_read_only() {
	test_file="$(pwd)/read_only_test.tmp"
	counter=0
	fail_count=0
	while [ $counter -lt 5 ]
	# Check touch command 5 times if it fails consistently. It might fail once or twice due to segmentation fault
	do
		counter=`expr $counter + 1`
		timeout -k 10 10 touch $test_file
		if [ $? -ne 0 ]; then
			fail_count=`expr $fail_count + 1`
		fi
		rm -f $test_file > /dev/null 2>&1
	done

	if [ $fail_count -eq 5 ]; then
		# Touch failed all 5 times so try fix sd card
		umount /dev/mmcblk0p1 > /dev/null 2>&1
		e2fsck -cy /dev/mmcblk0p1 > /dev/null 2>&1

		# Irrespective of above command status reboot now
		logger -t qikpod-system-check -p 3 "File system P1 seems to be locked, do reboot -f..."
		reboot -f
	fi
}

check_p2_read_only() {
	test_file="/root/data/read_only_test.tmp"
	counter=0
	fail_count=0
	while [ $counter -lt 5 ]
	# Check touch command 5 times if it fails consistently. It might fail once or twice due to segmentation fault
	do
		counter=`expr $counter + 1`
		timeout -k 10 10 touch $test_file
		if [ $? -ne 0 ]; then
			fail_count=`expr $fail_count + 1`
		fi
		rm -f $test_file > /dev/null 2>&1
	done

	if [ $fail_count -eq 5 ]; then
		# Touch failed all 5 times so try fix sd card
		monit quit
		/etc/init.d/qikpod stop
		umount /root/data > /dev/null 2>&1
		e2fsck -cy /dev/mmcblk0p2 > /dev/null 2>&1

		# Irrespective of above command status reboot now
		logger -t qikpod-system-check -p 3 "File system P2 seems to be locked, do reboot -f..."
		reboot -f
	fi
}

watch_file() {
	if [ -f $WATCH_DOG_FILE ]; then
		file_modification_time=`date -r $WATCH_DOG_FILE +%s`
		current_time=$(date +%s)
		time_diff=`expr $current_time - $file_modification_time`
		if [ $time_diff -gt 10800 ]; then
			# If not modified for last 3 hours reboot the system once
			log "QikPod service has not updated watch dog file for so long. Current time: $current_time, File Modified at: $file_modification_time. Rebooting..."
			reboot -f
		fi
	fi
}

watch_symlinks() {
    if [ ! -c "/dev/ttyUSBcomm" ] || [ ! -c "/dev/ttyUSBdata" ]; then
        log "Symlinks Missing"
        /etc/init.d/udev restart
        $(pwd)/monit/alert.sh "PROCESS_UDEV" "Symlinks Missing" "Restarted"
    fi
}

BASEDIR=$(dirname $0)
cd $BASEDIR
CWD=$(pwd)

while true
do
	log "Doing system check. File location: $CWD..."

	check_p1_read_only

	check_p2_read_only

	# watch_file

	# watch_symlinks


	# Check if emi occured in the pod
	EMI_LOG_FILE=$(pwd)/logs/emi_err.log
	LAST_REPORTED_COUNT=$(pwd)/logs/emi_err_count.log
	logread | grep -i emi >> $EMI_LOG_FILE
	sort -u $EMI_LOG_FILE > /tmp/emi.log
	cat /tmp/emi.log > $EMI_LOG_FILE
	emi_count=`wc -l < $EMI_LOG_FILE`
	if [ $emi_count -gt 0 ]; then
		if [ -f $LAST_REPORTED_COUNT ]; then
			last_count=`cat $LAST_REPORTED_COUNT`
			if [ $emi_count -ne $last_count ]; then
				log "EMI File exists, but count has changed from $last_count to $emi_count. Reporting to server..."
				$(pwd)/monit/alert.sh "EMI_COUNT" "$emi_count"
				echo "$emi_count" > $LAST_REPORTED_COUNT
			fi
		else
			log "EMI File not exists, pushing emi count to server..."
			$(pwd)/monit/alert.sh "EMI_COUNT" "$emi_count"
			echo "$emi_count" > $LAST_REPORTED_COUNT
		fi
	fi

	MAIN_PID=$(pgrep -f main.py)
	if [ "$?" -eq 0 ]; then

		# Check if multiple instances of main.py exist
		count=$(pgrep -f main.py | wc -l)
		if [ "$count" -gt 3 ]; then
			log "Qikpod service found running multiple times, reporting to server"
			$(pwd)/monit/alert.sh "MULTIPLE_PROCESS" "$count"
		fi
	fi
	if [[ $# -gt 0 ]] && [[ $1 == "once" ]]; then
    	log "System check called with run once option so exiting..."
    	break
	fi
	sleep 900 # Next run after 15 mins
done

cd -
