#!/usr/bin/python

from datetime import datetime
from logging.handlers import RotatingFileHandler
import logging
import os
import sys
import time
import rtc
import commands

abs_path = os.getcwd()

BACKUP_COUNT = 10

MAX_LOG_SIZE = 5 * 1024 * 1024

LOGGER = logging.getLogger(__name__)

hdlr = RotatingFileHandler(abs_path+'/logs/monit.log', maxBytes=MAX_LOG_SIZE, backupCount=BACKUP_COUNT)
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
LOGGER.addHandler(hdlr)
LOGGER.setLevel(logging.INFO)

FUTURE_TIME_THRESHHOLD = 5 * 12 * 30 * 24 * 60 * 60  # Approx 5 yrs
NTP_SERVERS = ['0.asia.pool.ntp.org', '1.asia.pool.ntp.org', '2.asia.pool.ntp.org', '3.asia.pool.ntp.org']
TIME_SYNC_FILE = "/tmp/qikpod_time_sync"


def sync_system_with_rtc():
    LOGGER.info("Syncing system time with RTC...")
    try:
        if rtc.is_time_date_valid():
            current_rtc = rtc.read_date_time()
            date_cmd = "timeout -k 10 15 date -s %d.%d.%d-%d:%d:%d" % (
                2000 + current_rtc.years, current_rtc.months, current_rtc.days, current_rtc.hours,
                current_rtc.mins, current_rtc.secs)
            status = os.system(date_cmd)
            if status == 0:
                return True
        else:
            LOGGER.warn("RTC Not Initialized or contains invalid time")
    except Exception:
        LOGGER.exception("Failed to initialize system time from RTC")

    LOGGER.warn("System time not synced with RTC")
    return False


def check_ping():
    try:
        status = os.system("timeout -k 10 20 ping -c 1 -W 10 8.8.8.8")
        if status == 0:
            return True
    except Exception:
        pass
    return False


def sync_system_with_ntp():
    LOGGER.info("Syncing system time with NTP...")

    if not check_ping():
        LOGGER.warn("No internet connection so skipping time sync")
        return False

    try:
        old_time = int(time.time())
        for server in NTP_SERVERS:
            try:
                status, output = commands.getstatusoutput("timeout -k 10 25 ntpdate {}".format(server))
                if status != 0:
                    LOGGER.warn("Error in syncing time with server {}: {}".format(server, output))
                    continue

                new_time = int(time.time())
                if new_time < old_time or new_time > old_time + FUTURE_TIME_THRESHHOLD:
                    LOGGER.error(
                        "Invalid time received from NTP: {}, when current time is: {}".format(new_time, old_time))
                    LOGGER.info("Syncing time again...")
                    continue
                open(TIME_SYNC_FILE, 'a').close()
                LOGGER.info("Time synced with server: '{}' and status: '{}'".format(server, output))
                return True
            except Exception:
                LOGGER.exception("Exception in syncing time with server: {}".format(server))
                continue
        LOGGER.warn("Unable to set system time with NTP")
    except Exception:
        LOGGER.exception("Failed to initialize system time from NTP")

    return False


def sync_rtc_with_system():
    LOGGER.info("Syncing RTC time with system...")
    now = datetime.now()

    days = now.day
    months = now.month
    years = now.year % 2000
    hours = now.hour
    mins = now.minute
    secs = now.second
    dow = now.isoweekday()

    current_rtc = rtc.RTC(secs, mins, hours, days, dow, months, years)

    rtc.set_date_time(current_rtc)
    return True


def main(args=None):
    if args is None or len(args) < 3:
        sync_system_with_rtc()

        ntp_status = sync_system_with_ntp()

        if ntp_status:
            sync_rtc_with_system()

        return ntp_status
    else:
        if args[1] == "rtc" and args[2] == "system":
            return sync_system_with_rtc()
        elif args[1] == "ntp" and args[2] == "system":
            return sync_system_with_ntp()
        elif args[1] == "system" and args[2] == "rtc":
            return sync_rtc_with_system()


if __name__ == '__main__':
    main(sys.argv)
